@@ -367,6 +367,8 @@ KateNavigationConfigTab::KateNavigationConfigTab(QWidget *parent)
     ui = new Ui::NavigationConfigWidget();
     ui->setupUi(newWidget);
 
+    initMulticursorModifierComboBox();
+
     // "What's This?" help can be found in the ui file
 
     reload();
@@ -378,6 +380,7 @@ KateNavigationConfigTab::KateNavigationConfigTab(QWidget *parent)
     observeChanges(ui->chkSmartHome);
     observeChanges(ui->sbAutoCenterCursor);
     observeChanges(ui->chkCamelCursor);
+    observeChanges(ui->cmbMultiCursorModifier);
 
     layout->addWidget(newWidget);
 }
@@ -387,6 +390,29 @@ KateNavigationConfigTab::~KateNavigationConfigTab()
     delete ui;
 }
 
+void KateNavigationConfigTab::initMulticursorModifierComboBox()
+{
+    // On macOS, the ControlModifier value corresponds to the Command keys on the
+    // keyboard, and the MetaModifier value corresponds to the Control keys.
+    auto *c = ui->cmbMultiCursorModifier;
+
+#ifndef Q_OS_DARWIN
+    c->insertItem(0, i18n("Alt"), (int)Qt::ALT);
+    c->insertItem(1, i18n("Ctrl"), (int)Qt::CTRL);
+    c->insertItem(2, i18n("Meta"), (int)Qt::META);
+    c->insertItem(3, i18n("Ctrl + Alt"), (int)(Qt::CTRL | Qt::ALT));
+    c->insertItem(4, i18n("Meta + Alt"), (int)(Qt::META | Qt::ALT));
+    c->insertItem(5, i18n("Ctrl + Meta"), (int)(Qt::CTRL | Qt::META));
+#else
+    c->insertItem(0, i18n("Alt"), (int)Qt::ALT);
+    c->insertItem(1, i18n("Cmd"), (int)Qt::CTRL);
+    c->insertItem(2, i18n("Ctrl"), (int)Qt::META);
+    c->insertItem(3, i18n("Cmd + Alt"), (int)(Qt::CTRL | Qt::ALT));
+    c->insertItem(4, i18n("Ctrl + Alt"), (int)(Qt::META | Qt::ALT));
+    c->insertItem(5, i18n("Cmd + Ctrl"), (int)(Qt::CTRL | Qt::META));
+#endif
+}
+
 void KateNavigationConfigTab::apply()
 {
     // nothing changed, no need to apply stuff
@@ -407,6 +433,9 @@ void KateNavigationConfigTab::apply()
     KateViewConfig::global()->setValue(KateViewConfig::PersistentSelection, ui->cbTextSelectionMode->currentIndex() == 1);
     KateViewConfig::global()->setValue(KateViewConfig::ScrollPastEnd, ui->chkScrollPastEnd->isChecked());
 
+    const int modifers = ui->cmbMultiCursorModifier->currentData().toInt();
+    KateViewConfig::global()->setMultiCursorModifiers(Qt::KeyboardModifiers(modifers));
+
     KateDocumentConfig::global()->configEnd();
     KateViewConfig::global()->configEnd();
 }
@@ -422,6 +451,16 @@ void KateNavigationConfigTab::reload()
     ui->chkCamelCursor->setChecked(KateDocumentConfig::global()->camelCursor());
 
     ui->sbAutoCenterCursor->setValue(KateViewConfig::global()->autoCenterLines());
+
+    const int mods = KateViewConfig::global()->multiCursorModifiers();
+    const auto count = ui->cmbMultiCursorModifier->count();
+    for (int i = 0; i < count; ++i) {
+        int idxMods = ui->cmbMultiCursorModifier->itemData(i).toInt();
+        if (idxMods == mods) {
+            ui->cmbMultiCursorModifier->setCurrentIndex(i);
+            break;
+        }
+    }
 }
 
 QString KateNavigationConfigTab::name() const
@@ -707,6 +746,7 @@ KateViewDefaultsConfig::KateViewDefaultsConfig(QWidget *parent)
     observeChanges(textareaUi->cmbDynamicWordWrapIndicator);
     observeChanges(textareaUi->cbxWordWrap);
     observeChanges(textareaUi->chkFocusFrame);
+    observeChanges(textareaUi->spbLineHeightMultiplier);
     auto a = [ui = textareaUi, cbx = textareaUi->cbxWordWrap]() {
         ui->chkDynWrapAtStaticMarker->setEnabled(cbx->isChecked());
         ui->chkDynWrapAnywhere->setEnabled(cbx->isChecked());
@@ -739,6 +779,7 @@ KateViewDefaultsConfig::KateViewDefaultsConfig(QWidget *parent)
     observeChanges(bordersUi->rbSortBookmarksByCreation);
     observeChanges(bordersUi->rbSortBookmarksByPosition);
     observeChanges(bordersUi->spBoxMiniMapWidth);
+    observeChanges(bordersUi->cmbFoldingArrowVisiblity);
 }
 
 KateViewDefaultsConfig::~KateViewDefaultsConfig()
@@ -766,6 +807,7 @@ void KateViewDefaultsConfig::apply()
     KateRendererConfig::global()->setAnimateBracketMatching(textareaUi->chkAnimateBracketMatching->isChecked());
     KateRendererConfig::global()->setShowIndentationLines(textareaUi->chkShowIndentationLines->isChecked());
     KateRendererConfig::global()->setShowWholeBracketExpression(textareaUi->chkShowWholeBracketExpression->isChecked());
+    KateRendererConfig::global()->setLineHeightMultiplier(textareaUi->spbLineHeightMultiplier->value());
 
     KateViewConfig::global()->setDynWordWrap(textareaUi->cbxWordWrap->isChecked());
     KateViewConfig::global()->setShowWordCount(textareaUi->chkShowWordCount->isChecked());
@@ -793,6 +835,8 @@ void KateViewDefaultsConfig::apply()
     KateViewConfig::global()->setValue(KateViewConfig::ShowScrollBarMiniMapAll, bordersUi->chkScrollbarMiniMapAll->isChecked());
     KateViewConfig::global()->setValue(KateViewConfig::ShowScrollBarPreview, bordersUi->chkScrollbarPreview->isChecked());
     KateViewConfig::global()->setValue(KateViewConfig::ShowScrollbars, bordersUi->cmbShowScrollbars->currentIndex());
+    bool showOnHoverOnly = bordersUi->cmbFoldingArrowVisiblity->currentIndex() == 0;
+    KateViewConfig::global()->setValue(KateViewConfig::ShowFoldingOnHoverOnly, showOnHoverOnly);
 
     KateRendererConfig::global()->configEnd();
     KateViewConfig::global()->configEnd();
@@ -813,6 +857,7 @@ void KateViewDefaultsConfig::reload()
     bordersUi->rbSortBookmarksByCreation->setChecked(KateViewConfig::global()->bookmarkSort() == 1);
     bordersUi->rbSortBookmarksByPosition->setChecked(KateViewConfig::global()->bookmarkSort() == 0);
     bordersUi->spBoxMiniMapWidth->setValue(KateViewConfig::global()->scrollBarMiniMapWidth());
+    bordersUi->cmbFoldingArrowVisiblity->setCurrentIndex(KateViewConfig::global()->showFoldingOnHoverOnly() ? 0 : 1);
 
     textareaUi->kfontrequester->setFont(KateRendererConfig::global()->baseFont());
 
@@ -833,6 +878,7 @@ void KateViewDefaultsConfig::reload()
     textareaUi->sliSetMarkerSize->setValue(KateDocumentConfig::global()->markerSize());
     textareaUi->spacesComboBox->setCurrentIndex(KateDocumentConfig::global()->showSpaces());
     textareaUi->chkFocusFrame->setChecked(KateViewConfig::global()->showFocusFrame());
+    textareaUi->spbLineHeightMultiplier->setValue(KateRendererConfig::global()->lineHeightMultiplier());
 }
 
 void KateViewDefaultsConfig::reset()
@@ -899,6 +945,11 @@ KateSaveConfigTab::KateSaveConfigTab(QWidget *parent)
     observeChanges(ui->cmbEncodingDetection);
     observeChanges(ui->cmbEncodingFallback);
     observeChanges(ui->lineLengthLimit);
+    observeChanges(ui->gbAutoSave);
+    observeChanges(ui->cbAutoSaveOnFocus);
+    observeChanges(ui->spbAutoSaveInterval);
+
+    observeChanges(uiadv->chkAutoReloadVersionControl);
 
     observeChanges(uiadv->chkBackupLocalFiles);
     observeChanges(uiadv->chkBackupRemoteFiles);
@@ -1009,6 +1060,12 @@ void KateSaveConfigTab::apply()
 
     KateDocumentConfig::global()->setLineLengthLimit(ui->lineLengthLimit->value());
 
+    KateDocumentConfig::global()->setValue(KateDocumentConfig::AutoSave, ui->gbAutoSave->isChecked());
+    KateDocumentConfig::global()->setValue(KateDocumentConfig::AutoSaveOnFocusOut, ui->cbAutoSaveOnFocus->isChecked());
+    KateDocumentConfig::global()->setValue(KateDocumentConfig::AutoSaveInteral, ui->spbAutoSaveInterval->value());
+
+    KateDocumentConfig::global()->setValue(KateDocumentConfig::AutoReloadIfStateIsInVersionControl, uiadv->chkAutoReloadVersionControl->isChecked());
+
     KateDocumentConfig::global()->configEnd();
     KateGlobalConfig::global()->configEnd();
 }
@@ -1076,6 +1133,12 @@ void KateSaveConfigTab::reload()
     uiadv->kurlSwapDirectory->setUrl(QUrl::fromLocalFile(KateDocumentConfig::global()->swapDirectory()));
     uiadv->spbSwapFileSync->setValue(KateDocumentConfig::global()->swapSyncInterval());
     swapFileModeChanged(KateDocumentConfig::global()->swapFileMode());
+
+    ui->gbAutoSave->setChecked(KateDocumentConfig::global()->autoSave());
+    ui->cbAutoSaveOnFocus->setChecked(KateDocumentConfig::global()->autoSaveOnFocusOut());
+    ui->spbAutoSaveInterval->setValue(KateDocumentConfig::global()->autoSaveInterval());
+
+    uiadv->chkAutoReloadVersionControl->setChecked(KateDocumentConfig::global()->value(KateDocumentConfig::AutoReloadIfStateIsInVersionControl).toBool());
 }
 
 void KateSaveConfigTab::reset()
