@@ -1,27 +1,14 @@
-/*  SPDX-License-Identifier: LGPL-2.0-or-later
+/*
+    SPDX-FileCopyrightText: 2005-2006 Hamish Rodda <rodda@kde.org>
 
-    Copyright (C) 2005-2006 Hamish Rodda <rodda@kde.org>
-
-    This library is free software; you can redistribute it and/or
-    modify it under the terms of the GNU Library General Public
-    License as published by the Free Software Foundation; either
-    version 2 of the License, or (at your option) any later version.
-
-    This library is distributed in the hope that it will be useful,
-    but WITHOUT ANY WARRANTY; without even the implied warranty of
-    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
-    Library General Public License for more details.
-
-    You should have received a copy of the GNU Library General Public License
-    along with this library; see the file COPYING.LIB.  If not, write to
-    the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
-    Boston, MA 02110-1301, USA.
+    SPDX-License-Identifier: LGPL-2.0-or-later
 */
 
 #ifndef KTEXTEDITOR_CODECOMPLETIONINTERFACE_H
 #define KTEXTEDITOR_CODECOMPLETIONINTERFACE_H
 
 #include <QObject>
+#include <ktexteditor/codecompletionmodel.h>
 #include <ktexteditor/range.h>
 #include <ktexteditor_export.h>
 
@@ -30,6 +17,8 @@ namespace KTextEditor
 class CodeCompletionModel;
 
 /**
+ * \class CodeCompletionInterface codecompletioninterface.h <KTextEditor/CodeCompletionInterface>
+ *
  * \brief Code completion extension interface for the View.
  *
  * \ingroup kte_group_view_extensions
@@ -150,8 +139,63 @@ public:
     // void completionAborted(KTextEditor::View* view);
 };
 
+/**
+ * \class CodeCompletionInterfaceV2 codecompletioninterface.h <KTextEditor/CodeCompletionInterface>
+ *
+ * \brief Code completion extension interface for the View, version 2.
+ *
+ * \ingroup kte_group_view_extensions
+ *
+ * \section compifacev2_intro Introduction
+ *
+ * The CodeCompletionInterfaceV2 is an extended version of the CodeCompletionInterface.
+ * Refer to the base CodeCompletionInterface for a more detailed description.
+ *
+ * \section compifacev2_access Accessing the CodeCompletionInterfaceV2
+ *
+ * The CodeCompletionInterfaceV2 is an extension interface for a
+ * View, i.e. the View inherits the interface \e provided that the
+ * used KTextEditor library implements the interface. Use qobject_cast to
+ * access the interface:
+ * \code
+ * // view is of type KTextEditor::View*
+ * auto iface = qobject_cast<KTextEditor::CodeCompletionInterfaceV2*>(view);
+ *
+ * if (iface) {
+ *     // the implementation supports the interface
+ *     // do stuff
+ * } else {
+ *     // the implementation does not support the interface
+ * }
+ * \endcode
+ *
+ * \since 5.74
+ */
+class KTEXTEDITOR_EXPORT CodeCompletionInterfaceV2 : public CodeCompletionInterface
+{
+    // KF6: Merge KTextEditor::CodeCompletionInterfaceV2 into KTextEditor::CodeCompletionInterface
+public:
+    ~CodeCompletionInterfaceV2() override;
+    /**
+     * Invoke code completion over a given range, with specific models and invocation type.
+     * \param models list of models to start. If this is an empty list, all registered models are started.
+     */
+    virtual void startCompletion(const Range &word,
+                                 const QList<CodeCompletionModel *> &models = QList<CodeCompletionModel *>(),
+                                 CodeCompletionModel::InvocationType invocationType = CodeCompletionModel::ManualInvocation) = 0;
+    using CodeCompletionInterface::startCompletion;
+
+    /**
+     * Obtain the list of registered code completion models.
+     * \returns a list of a models that are currently registered
+     * \see registerCompletionModel(CodeCompletionModel*)
+     */
+    virtual QList<CodeCompletionModel *> codeCompletionModels() const = 0;
+};
+
 }
 
 Q_DECLARE_INTERFACE(KTextEditor::CodeCompletionInterface, "org.kde.KTextEditor.CodeCompletionInterface")
+Q_DECLARE_INTERFACE(KTextEditor::CodeCompletionInterfaceV2, "org.kde.KTextEditor.CodeCompletionInterfaceV2")
 
 #endif // KTEXTEDITOR_CODECOMPLETIONINTERFACE_H
