@@ -150,6 +150,23 @@ KateSearchBar::KateSearchBar(bool initAsPower, KTextEditor::ViewPrivate *view, K
     connect(view, &KTextEditor::View::selectionChanged, this, &KateSearchBar::updateSelectionOnly);
     connect(this, &KateSearchBar::findOrReplaceAllFinished, this, &KateSearchBar::endFindOrReplaceAll);
 
+    auto setSelectionChangedByUndoRedo = [this]() {
+        m_selectionChangedByUndoRedo = true;
+    };
+    auto unsetSelectionChangedByUndoRedo = [this]() {
+        m_selectionChangedByUndoRedo = false;
+    };
+    KateUndoManager *docUndoManager = view->doc()->undoManager();
+    connect(docUndoManager, &KateUndoManager::undoStart, this, setSelectionChangedByUndoRedo);
+    connect(docUndoManager, &KateUndoManager::undoEnd, this, unsetSelectionChangedByUndoRedo);
+    connect(docUndoManager, &KateUndoManager::redoStart, this, setSelectionChangedByUndoRedo);
+    connect(docUndoManager, &KateUndoManager::redoEnd, this, unsetSelectionChangedByUndoRedo);
+
+    // When document is reloaded, disable selection-only search so that the search won't be stuck after the first search
+    connect(view->doc(), &KTextEditor::Document::reloaded, this, [this]() {
+        setSelectionOnly(false);
+    });
+
     // init match attribute
     Attribute::Ptr mouseInAttribute(new Attribute());
     mouseInAttribute->setFontBold(true);
@@ -206,6 +223,9 @@ KateSearchBar::~KateSearchBar()
 
     delete m_incUi;
     delete m_powerUi;
+    if (m_workingRange) {
+        delete m_workingRange;
+    }
 }
 
 void KateSearchBar::closed()
@@ -291,7 +311,7 @@ void KateSearchBar::showResultMessage()
     }
 }
 
-void KateSearchBar::highlightMatch(const Range &range)
+void KateSearchBar::highlightMatch(Range range)
 {
     KTextEditor::MovingRange *const highlight = m_view->doc()->newMovingRange(range, Kate::TextRange::DoNotExpand);
     highlight->setView(m_view); // show only in this view
@@ -302,7 +322,7 @@ void KateSearchBar::highlightMatch(const Range &range)
     m_hlRanges.append(highlight);
 }
 
-void KateSearchBar::highlightReplacement(const Range &range)
+void KateSearchBar::highlightReplacement(Range range)
 {
     KTextEditor::MovingRange *const highlight = m_view->doc()->newMovingRange(range, Kate::TextRange::DoNotExpand);
     highlight->setView(m_view); // show only in this view
@@ -370,13 +390,13 @@ void KateSearchBar::indicateMatch(MatchResult matchResult)
     lineEdit->setPalette(background);
 }
 
-/*static*/ void KateSearchBar::selectRange(KTextEditor::ViewPrivate *view, const KTextEditor::Range &range)
+/*static*/ void KateSearchBar::selectRange(KTextEditor::ViewPrivate *view, KTextEditor::Range range)
 {
     view->setCursorPositionInternal(range.end());
     view->setSelection(range);
 }
 
-void KateSearchBar::selectRange2(const KTextEditor::Range &range)
+void KateSearchBar::selectRange2(KTextEditor::Range range)
 {
     disconnect(m_view, &KTextEditor::View::selectionChanged, this, &KateSearchBar::updateSelectionOnly);
     selectRange(m_view, range);
@@ -493,8 +513,25 @@ bool KateSearchBar::findOrReplace(SearchDirection searchDirection, const QString
     const Range selection = m_view->selection() ? m_view->selectionRange() : Range::invalid();
     if (selection.isValid()) {
         if (selectionOnly()) {
-            // First match in selection
-            inputRange = selection;
+            if (m_workingRange == nullptr) {
+                m_workingRange =
+                    m_view->doc()->newMovingRange(KTextEditor::Range::invalid(), KTextEditor::MovingRange::ExpandLeft | KTextEditor::MovingRange::ExpandRight);
+            }
+            if (!m_workingRange->toRange().isValid()) {
+                // First match in selection
+                inputRange = selection;
+                // Remember selection for succeeding selection-only searches
+                // Elsewhere, make sure m_workingRange is invalidated when selection/search range changes
+                m_workingRange->setRange(selection);
+            } else {
+                // The selection wasn't changed/updated by user, so we use the previous selection
+                // We use the selection's start/end so that the search can move forward/backward
+                if (searchDirection == SearchBackward) {
+                    inputRange.setRange(m_workingRange->start(), selection.end());
+                } else {
+                    inputRange.setRange(selection.start(), m_workingRange->end());
+                }
+            }
         } else {
             // Next match after/before selection if a match was selected before
             if (searchDirection == SearchForward) {
@@ -502,9 +539,16 @@ bool KateSearchBar::findOrReplace(SearchDirection searchDirection, const QString
             } else {
                 inputRange.setRange(Cursor(0, 0), selection.end());
             }
+
+            // Discard selection/search range previously remembered
+            if (m_workingRange) {
+                delete m_workingRange;
+                m_workingRange = nullptr;
+            }
         }
     } else {
         // No selection
+        setSelectionOnly(false);
         const Cursor cursorPos = m_view->cursorPosition();
         if (searchDirection == SearchForward) {
             inputRange.setRange(cursorPos, m_view->document()->documentEnd());
@@ -531,15 +575,13 @@ bool KateSearchBar::findOrReplace(SearchDirection searchDirection, const QString
                 delete smartInputRange;
             }
 
-            if (!selectionOnly()) {
-                // Find, second try after old selection
-                if (searchDirection == SearchForward) {
-                    const Cursor start = (replacement != nullptr) ? afterReplace.end() : selection.end();
-                    inputRange.setRange(start, inputRange.end());
-                } else {
-                    const Cursor end = (replacement != nullptr) ? afterReplace.start() : selection.start();
-                    inputRange.setRange(inputRange.start(), end);
-                }
+            // Find, second try after old selection
+            if (searchDirection == SearchForward) {
+                const Cursor start = (replacement != nullptr) ? afterReplace.end() : selection.end();
+                inputRange.setRange(start, inputRange.end());
+            } else {
+                const Cursor end = (replacement != nullptr) ? afterReplace.start() : selection.start();
+                inputRange.setRange(inputRange.start(), end);
             }
 
             match.searchText(inputRange, searchPattern());
@@ -561,9 +603,8 @@ bool KateSearchBar::findOrReplace(SearchDirection searchDirection, const QString
         }
     }
 
-    bool askWrap = !match.isValid() && (!selection.isValid() || !selectionOnly());
+    bool askWrap = !match.isValid() && (!afterReplace.isValid() || !selectionOnly());
     bool wrap = false;
-
     if (askWrap) {
         askWrap = false;
         wrap = true;
@@ -582,7 +623,11 @@ bool KateSearchBar::findOrReplace(SearchDirection searchDirection, const QString
     }
     if (wrap) {
         m_view->showSearchWrappedHint(searchDirection == SearchBackward);
-        inputRange = m_view->document()->documentRange();
+        if (selectionOnly() && m_workingRange != nullptr && m_workingRange->toRange().isValid()) {
+            inputRange = m_workingRange->toRange();
+        } else {
+            inputRange = m_view->document()->documentRange();
+        }
         match.searchText(inputRange, searchPattern());
     }
 
@@ -765,18 +810,24 @@ void KateSearchBar::findOrReplaceAll()
 
     int line = m_inputRange.start().line();
 
-    QElapsedTimer rolex; // Watchog to suspend the work after some time
-    rolex.start();
     bool timeOut = false;
     bool done = false;
+
+    // This variable holds the number of lines that we have searched
+    // When it reaches 50K, we break the loop to allow event processing
+    int numLinesSearched = 0;
+    // Use a simple range in the loop to avoid needless work
+    KTextEditor::Range workingRangeCopy = m_workingRange->toRange();
     do {
         if (block) {
             delete m_workingRange; // Never forget that!
             m_workingRange = m_view->doc()->newMovingRange(m_view->doc()->rangeOnLine(m_inputRange, line));
+            workingRangeCopy = m_workingRange->toRange();
         }
 
         do {
-            match.searchText(*m_workingRange, searchPattern());
+            int currentSearchLine = workingRangeCopy.start().line();
+            match.searchText(workingRangeCopy, searchPattern());
             if (!match.isValid()) {
                 done = true;
                 break;
@@ -792,6 +843,8 @@ void KateSearchBar::findOrReplaceAll()
 
                 // Replace
                 lastRange = match.replace(m_replacement, false, ++m_matchCounter);
+                // update working range as text must have changed now
+                workingRangeCopy = m_workingRange->toRange();
             } else {
                 lastRange = match.range();
                 ++m_matchCounter;
@@ -806,7 +859,7 @@ void KateSearchBar::findOrReplaceAll()
             }
 
             // Continue after match
-            if (lastRange.end() >= m_workingRange->end()) {
+            if (lastRange.end() >= workingRangeCopy.end()) {
                 done = true;
                 break;
             }
@@ -818,20 +871,26 @@ void KateSearchBar::findOrReplaceAll()
                 // If we don't advance here we will loop forever...
                 workingStart.move(1);
             }
-            m_workingRange->setRange(workingStart.toCursor(), m_workingRange->end());
+            workingRangeCopy.setRange(workingStart.toCursor(), workingRangeCopy.end());
 
             // Are we done?
-            if (!m_workingRange->toRange().isValid() || workingStart.atEndOfDocument()) {
+            if (!workingRangeCopy.isValid() || workingStart.atEndOfDocument()) {
                 done = true;
                 break;
             }
 
-            timeOut = rolex.elapsed() > 150;
+            // Check if we have searched through 50K lines and time out.
+            // We do this to allow the search operation to be cancelled
+            numLinesSearched += workingRangeCopy.start().line() - currentSearchLine;
+            timeOut = numLinesSearched >= 50000;
 
         } while (!m_cancelFindOrReplace && !timeOut);
 
     } while (!m_cancelFindOrReplace && !timeOut && block && ++line <= m_inputRange.end().line());
 
+    // update m_workingRange
+    m_workingRange->setRange(workingRangeCopy);
+
     if (done || m_cancelFindOrReplace) {
         Q_EMIT findOrReplaceAllFinished();
     } else if (timeOut) {
@@ -867,14 +926,14 @@ void KateSearchBar::endFindOrReplaceAll()
 
     // Add highlights
     if (m_replaceMode) {
-        for (const Range &r : qAsConst(m_highlightRanges)) {
+        for (const Range &r : std::as_const(m_highlightRanges)) {
             highlightReplacement(r);
         }
         // Never merge replace actions with other replace actions/user actions
         m_view->doc()->undoManager()->undoSafePoint();
 
     } else {
-        for (const Range &r : qAsConst(m_highlightRanges)) {
+        for (const Range &r : std::as_const(m_highlightRanges)) {
             highlightMatch(r);
         }
         //         indicateMatch(m_matchCounter > 0 ? MatchFound : MatchMismatch); TODO
@@ -882,6 +941,7 @@ void KateSearchBar::endFindOrReplaceAll()
 
     // Clean-Up the still hold MovingRange
     delete m_workingRange;
+    m_workingRange = nullptr; // m_workingRange is also used elsewhere so we signify that it is now "unused"
 
     // restore connection
     connect(m_view, &KTextEditor::View::selectionChanged, this, &KateSearchBar::updateSelectionOnly);
@@ -1127,7 +1187,7 @@ void KateSearchBar::showExtendedContextMenu(bool forPattern, const QPoint &pos)
                 for (int i = 1; i <= 9; i++) {
                     const QString number = QString::number(i);
                     const QString &captureDetails =
-                        (i <= captureCount) ? QLatin1String(" = (") + capturePatterns[i - 1].leftRef(30) + QLatin1Char(')') : QString();
+                        (i <= captureCount) ? QLatin1String(" = (") + QStringView(capturePatterns[i - 1]).left(30) + QLatin1Char(')') : QString();
                     addMenuManager.addEntry(QLatin1String("\\") + number, QString(), i18n("Reference") + QLatin1Char(' ') + number + captureDetails);
                 }
 
@@ -1195,6 +1255,11 @@ void KateSearchBar::onPowerModeChanged(int /*index*/)
     givePatternFeedback();
 }
 
+// static void addSecondarySelection(KTextEditor::ViewPrivate *view, KTextEditor::Range range)
+// {
+//     view->addSecondaryCursorWithSelection(range);
+// }
+
 void KateSearchBar::nextMatchForSelection(KTextEditor::ViewPrivate *view, SearchDirection searchDirection)
 {
     if (!view->selection()) {
@@ -1207,7 +1272,8 @@ void KateSearchBar::nextMatchForSelection(KTextEditor::ViewPrivate *view, Search
         }
     }
     if (view->selection()) {
-        const QString pattern = view->selectionText();
+        // We only want text of one of the selection ranges
+        const QString pattern = view->doc()->text(view->selectionRange());
 
         // How to find?
         SearchOptions enabledOptions(KTextEditor::Default);
@@ -1217,6 +1283,7 @@ void KateSearchBar::nextMatchForSelection(KTextEditor::ViewPrivate *view, Search
 
         // Where to find?
         const Range selRange = view->selectionRange();
+        //         const Range selRange = view->lastSelectionRange();
         Range inputRange;
         if (searchDirection == SearchForward) {
             inputRange.setRange(selRange.end(), view->doc()->documentEnd());
@@ -1230,6 +1297,7 @@ void KateSearchBar::nextMatchForSelection(KTextEditor::ViewPrivate *view, Search
 
         if (match.isValid()) {
             selectRange(view, match.range());
+            //             addSecondarySelection(view, match.range());
         } else {
             // Find, second try
             m_view->showSearchWrappedHint(searchDirection == SearchBackward);
@@ -1242,6 +1310,7 @@ void KateSearchBar::nextMatchForSelection(KTextEditor::ViewPrivate *view, Search
             match2.searchText(inputRange, pattern);
             if (match2.isValid()) {
                 selectRange(view, match2.range());
+                //                 addSecondarySelection(view, match2.range());
             }
         }
     }
@@ -1378,7 +1447,7 @@ void KateSearchBar::enterPowerMode()
         connect(m_powerUi->findPrev, &QToolButton::clicked, this, &KateSearchBar::findPrevious);
         connect(m_powerUi->replaceNext, &QPushButton::clicked, this, &KateSearchBar::replaceNext);
         connect(m_powerUi->replaceAll, &QPushButton::clicked, this, &KateSearchBar::replaceAll);
-        connect(m_powerUi->searchMode, QOverload<int>::of(&QComboBox::currentIndexChanged), this, &KateSearchBar::onPowerModeChanged);
+        connect(m_powerUi->searchMode, qOverload<int>(&QComboBox::currentIndexChanged), this, &KateSearchBar::onPowerModeChanged);
         connect(m_powerUi->matchCase, &QToolButton::toggled, this, &KateSearchBar::onMatchCaseToggled);
         connect(m_powerUi->findAll, &QPushButton::clicked, this, &KateSearchBar::findAll);
         connect(m_powerUi->cancel, &QPushButton::clicked, this, &KateSearchBar::onPowerCancelFindOrReplace);
@@ -1390,21 +1459,21 @@ void KateSearchBar::enterPowerMode()
         // Hook into line edit context menus
         m_powerUi->pattern->setContextMenuPolicy(Qt::CustomContextMenu);
 
-        connect(m_powerUi->pattern,
-                &QComboBox::customContextMenuRequested,
-                this,
-                QOverload<const QPoint &>::of(&KateSearchBar::onPowerPatternContextMenuRequest));
+        connect(m_powerUi->pattern, &QComboBox::customContextMenuRequested, this, qOverload<const QPoint &>(&KateSearchBar::onPowerPatternContextMenuRequest));
         m_powerUi->replacement->setContextMenuPolicy(Qt::CustomContextMenu);
         connect(m_powerUi->replacement,
                 &QComboBox::customContextMenuRequested,
                 this,
-                QOverload<const QPoint &>::of(&KateSearchBar::onPowerReplacmentContextMenuRequest));
+                qOverload<const QPoint &>(&KateSearchBar::onPowerReplacmentContextMenuRequest));
     }
 
     // Focus
     if (m_widget->isVisible()) {
         m_powerUi->pattern->setFocus(Qt::MouseFocusReason);
     }
+
+    // move close button to right layout, ensures properly at top for both incremental + advanced mode
+    m_powerUi->gridLayout->addWidget(closeButton(), 0, 2, 1, 1);
 }
 
 void KateSearchBar::enterIncrementalMode()
@@ -1540,6 +1609,9 @@ void KateSearchBar::enterIncrementalMode()
     if (m_widget->isVisible()) {
         m_incUi->pattern->setFocus(Qt::MouseFocusReason);
     }
+
+    // move close button to right layout, ensures properly at top for both incremental + advanced mode
+    m_incUi->hboxLayout->addWidget(closeButton());
 }
 
 bool KateSearchBar::clearHighlights()
@@ -1595,7 +1667,12 @@ void KateSearchBar::showEvent(QShowEvent *event)
         m_incInitCursor = m_view->cursorPosition();
     }
 
-    updateSelectionOnly();
+    // We don't want to update if a "findOrReplaceAll" is running
+    // other we end up deleting our working range and crash
+    if (m_cancelFindOrReplace) {
+        updateSelectionOnly();
+    }
+
     KateViewBarWidget::showEvent(event);
 }
 
@@ -1625,6 +1702,12 @@ bool KateSearchBar::eventFilter(QObject *obj, QEvent *event)
 
 void KateSearchBar::updateSelectionOnly()
 {
+    // Make sure the previous selection-only search range is not used anymore
+    if (m_workingRange && !m_selectionChangedByUndoRedo) {
+        delete m_workingRange;
+        m_workingRange = nullptr;
+    }
+
     if (m_powerUi == nullptr) {
         return;
     }
