@@ -10,7 +10,6 @@
 #include "katedocument.h"
 #include "katerenderer.h"
 #include "kateview.h"
-#include "ktexteditor/range.h"
 #include <vimode/inputmodemanager.h>
 #include <vimode/modes/modebase.h>
 
@@ -25,13 +24,13 @@ Searcher::Searcher(InputModeManager *manager)
     updateHighlightColors();
 
     if (m_hlMode == HighlightMode::Enable) {
-        connectDisplayRangeChanged();
+        connectSignals();
     }
 }
 
 Searcher::~Searcher()
 {
-    disconnectDisplayRangeChanged();
+    disconnectSignals();
     clearHighlights();
 }
 
@@ -176,7 +175,6 @@ void Searcher::highlightVisibleResults(const SearchParams &searchParams, bool fo
 
     KTextEditor::Range match;
     KTextEditor::Cursor current(vr.start());
-    KTextEditor::Cursor end(m_view->doc()->documentRange().end());
 
     do {
         match = m_view->doc()->searchText(KTextEditor::Range(current, vr.end()), pattern, flags).first();
@@ -193,7 +191,7 @@ void Searcher::highlightVisibleResults(const SearchParams &searchParams, bool fo
 
             current = match.end();
         }
-    } while (match.isValid() && current < end);
+    } while (match.isValid() && current < vr.end());
 }
 
 void Searcher::clearHighlights()
@@ -226,12 +224,12 @@ void Searcher::enableHighlightSearch(bool enable)
     if (enable) {
         m_hlMode = HighlightMode::Enable;
 
-        connectDisplayRangeChanged();
+        connectSignals();
         highlightVisibleResults(m_lastSearchConfig, true);
     } else {
         m_hlMode = HighlightMode::Disable;
 
-        disconnectDisplayRangeChanged();
+        disconnectSignals();
         clearHighlights();
     }
 }
@@ -241,18 +239,23 @@ bool Searcher::isHighlightSearchEnabled() const
     return m_hlMode != HighlightMode::Disable;
 }
 
-void Searcher::disconnectDisplayRangeChanged()
+void Searcher::disconnectSignals()
 {
     QObject::disconnect(m_displayRangeChangedConnection);
+    QObject::disconnect(m_textChangedConnection);
 }
 
-void Searcher::connectDisplayRangeChanged()
+void Searcher::connectSignals()
 {
-    disconnectDisplayRangeChanged();
+    disconnectSignals();
 
     m_displayRangeChangedConnection = QObject::connect(m_view, &KTextEditor::ViewPrivate::displayRangeChanged, [this]() {
         if (m_hlMode == HighlightMode::Enable)
-            highlightVisibleResults(m_lastSearchConfig);
+            highlightVisibleResults(m_lastHlSearchConfig);
+    });
+    m_textChangedConnection = QObject::connect(m_view->doc(), &KTextEditor::Document::textChanged, [this]() {
+        if (m_hlMode == HighlightMode::Enable)
+            highlightVisibleResults(m_lastHlSearchConfig, true);
     });
 }
 
